#!/bin/bash

# DocBook-create.sh
# (c) 2003 Dennis Leeuw
# License GPL, see COPYING

PROC="/proc/sys"
DESCRIPTION_DIR="descriptions.d"
DESCRIPTION_DIR_SYS="$DESCRIPTION_DIR/`uname -s`"
DESCRIPTION_DIR_PROC="$DESCRIPTION_DIR_SYS/proc"

# Functions
function create_config () {
	local ONE=$1

	# If a file is found corresponding to the /proc entry
	# The file is cat. If no such file is found then
	# it is looked at in dir/../default to see if such a file
	# exists to cover e.g. conf/eth0/file
	# And if that also does not exist. No description is printed.
	local PLACE=$1
	# Get the path
	local DIR=${PLACE%/*}
	local DIR_BASE=${DIR%/*}
	# Get the file
	local FILE=${PLACE##*/}
	if [ -f $DESCRIPTION_DIR_SYS/$DIR/$FILE ]; then
		local CONTENT=`cat $DESCRIPTION_DIR_SYS/$DIR/$FILE`
	elif [ -f $DESCRIPTION_DIR_SYS/$DIR_BASE/default/$FILE ]; then
		local CONTENT=`cat $DESCRIPTION_DIR_SYS/$DIR_BASE/default/$FILE`
	else
		local CONTENT="No description for ${FILE}"
	fi

	CONTENT="<formalpara><title>$FILE</title><para>${CONTENT}</para></formalpara>"
	echo $CONTENT
}

function create_header () {
	# This creates a section header starting with ='s
	# followed by the path names
	# followed by ='s
	# followed by the file name
	# followed by -'s
	local PLACE=$1
	local SYS_DIR=$DESCRIPTION_DIR_SYS
	# Replace all dir separaters by spaces
	local DIR_LIST=${PLACE//\// }
	local HEADER="Linux "
	local DIR=""
	for DIR in $DIR_LIST; do
		if [ -f $SYS_DIR/$DIR/base.title ]; then
			HEADER="$HEADER `cat $SYS_DIR/$DIR/base.title`"
		fi
		SYS_DIR=$SYS_DIR/$DIR
	done
	echo "<chapter><title>$HEADER</title>"
}

function do_config_request () {
	# This creates a header
	# Prints the base.desc file or No section description
	# And then asks the user if it want to configure anything in this
	# section
	
	if [ -f $DESCRIPTION_DIR_SYS/$1/base.desc ]; then
		local CONTENT=`cat $DESCRIPTION_DIR_SYS/$1/base.desc`
		local TITLE=`cat $DESCRIPTION_DIR_SYS/$1/base.title`
	else
		local CONTENT="No section description for $1"
		local TITLE=$1
	fi
	if [ $SECT = 1 ]; then
		echo "</sect1>"
	fi
	echo "<sect1><title>$TITLE</title>"
	echo "<para>$CONTENT</para>"
	SECT=1
	fstate `echo $1/*`
}

function fstate () {
	local ONE=""
	for ONE in $@; do
		if [ -f $ONE ]; then
			# So it is a file. Let's see if it is writable.
			# Only if it is we want to set values :)
			# test -w doesn't seem to work reliable on /proc
			local CURRENT_VALUE=`cat $ONE`
			echo "$CURRENT_VALUE" 2>/dev/null > $ONE
			if [ $? = 0 ]; then
				create_config $ONE
			fi
		fi
	done
}

function dir_loop () {
	# Let's look inside the dir what's there
	# And feed that to fstate to determine what to do with it
	local ONE=0
	local ALL=`echo $1/*`
	for ONE in $ALL; do
		if [ -d $ONE ]; then
			DIR_INDEX=$(($DIR_INDEX+1))
			DIR_DB[$DIR_INDEX]=$ONE
			DIRS="$DIRS $ONE"
		fi
	done
	loop_dir_loop $DIRS
}

function loop_dir_loop () {
	local DIR=""
	DIRS=""
	for DIR in $@; do
		dir_loop $DIR
	done
}

# Main

# Program vars
declare DIR_DB
DIR_INDEX=0
DIRS=""

# For testing purposes to jump into a branch:
if [ "x$1" != "x" ]; then
	PROC="$PROC/$1"
fi


echo "<!doctype book public \"-//OASIS//DTD DocBook V3.1//EN\" []>"
echo "<chapter><title>Essential Linux terminology</title>"
echo "<formalpara><title>Jiffie</title>"
echo "<para>Internal timeunit for the kernel. On the i386 1/100s, on the Alpha 1/1024s. See the HZ define in /usr/include/asm/param.h for the exact value on your system.</para></formalpara>"
echo "</chapter>"
SECT=0

dir_loop $PROC
for (( I=1; I < ${#DIR_DB[*]}; I=$(($I+1)) )); do
	create_header ${DIR_DB[$I]}
	do_config_request ${DIR_DB[$I]}
	echo "</chapter>"
done

dir_loop $PROC

echo "</sect1></chapter></book>"
