#!/bin/bash

# haring.sh
# (c) 2003 Dennis Leeuw
# License GPL, see COPYING

# Sys vars
MOUNT_PROC=no
MAKE_PROC=no
PROC="/proc"

# Haring vars
HARING_DIR="/usr/local/haring-0.2.3"
DESCRIPTIONS="${HARING_DIR}/descriptions.d"
CONFIG_FILE="${HARING_DIR}/etc/kernel.conf"

# Functions
function set_params () {
	if [ -f $CONFIG_FILE ]; then
		echo -n "Setting up kernel configuration ."
		IFS=$'\n'
		for LINE in `cat $CONFIG_FILE`; do
			PARAM=${LINE%=*}
			VALUE=${LINE#*=}
	
			PARAM=${PARAM//\.//}
			if [ -w /$PARAM ]; then
				echo "$VALUE" 2>/dev/null > /$PARAM
				echo -n "."
			fi
		done
		echo " done."
	else
		echo "No $CONFIG_FILE"
		echo "Run haring.sh config"
	fi
}


# Main

# See if there is a proc mountpoint
if [ ! -d $PROC ]; then
	MAKE_PROC=yes
	echo "Creating $PROC mount point"
	mkdir $PROC
fi

# See if proc is mounted
MOUNTED_LIST=`mount`
MOUNTED_TEST=${MOUNTED_LIST/type proc/}
if [ ${#MOUNTED_LIST} = ${#MOUNTED_TEST} ]; then
	MOUNT_PROC=yes
	echo "Mounting $PROC"
	mount -t proc /proc
fi

# Now proc should be available to us.
case $1 in
	start) set_params
	;;
	config) ${HARING_DIR}/haring-config.sh
	;;
	*) echo "Syntax: ${HARING_DIR}/haring.sh [start|config]"
	   exit 1
esac

# Undo what we have done to get proc
if [ $MOUNT_PROC = yes ]; then
	echo "Unmounting $PROC"
	umount $PROC
fi

if [ $MAKE_PROC = yes ]; then
	echo "Removing the $PROC mount point"
	rmdir $PROC
fi
