#!/bin/bash

# haring-config.sh
# (c) 2003 Dennis Leeuw
# License GPL, see COPYING

PROC="/proc/sys"
DESCRIPTION_DIR="descriptions.d"
DESCRIPTION_DIR_SYS="$DESCRIPTION_DIR/`uname -s`"
DESCRIPTION_DIR_PROC="$DESCRIPTION_DIR_SYS/proc"

CONFIG_FILE="etc/kernel.conf"
if [ -f $CONFIG_FILE ]; then
	# If a file already exists, make a backup with YYYYMMDDhhmmss
	mv $CONFIG_FILE ${CONFIG_FILE}.pre-`date +%Y%m%d%k%m%S`
fi

# Functions
function create_config () {
	# Prints a header and the current value
	# It then asks for a new value.
	# The output is written to the CONFIG_FILE
	local ONE=$1
	create_header $ONE
	create_description $ONE
	echo -n "Current value: "; cat $ONE
	echo -n "Your value: "; read VALUE
	if [ "x$VALUE" = "x" ]; then
		VALUE=`cat $ONE`
	fi

	local PARAM=${ONE#/}
	echo "${PARAM//\//.}=$VALUE" >>$CONFIG_FILE
}

function create_description () {
	# If a file is found corresponding to the /proc entry
	# The file is cat. If no such file is found then
	# it is looked at in dir/../default to see if such a file
	# exists to cover e.g. conf/eth0/file
	# And if that also does not exist. No description is printed.
	local PLACE=$1
	# Get the path
	local DIR=${PLACE%/*}
	local DIR_BASE=${DIR%/*}
	# Get the file
	local FILE=${PLACE##*/}
	if [ -f $DESCRIPTION_DIR_SYS/$DIR/$FILE ]; then
		cat $DESCRIPTION_DIR_SYS/$DIR/$FILE
	elif [ -f $DESCRIPTION_DIR_SYS/$DIR_BASE/default/$FILE ]; then
		cat $DESCRIPTION_DIR_SYS/$DIR_BASE/default/$FILE
	else
		echo "*** No description for ${FILE}"
	fi
}

function create_header () {
	# This creates a section header starting with ='s
	# followed by the path names
	# followed by ='s
	# followed by the file name
	# followed by -'s
	clear
	echo "==============================================================================="
	local PLACE=$1
	local SYS_DIR=$DESCRIPTION_DIR_SYS
	# Strip the file
	local DIR_LIST=${PLACE%/*}
	# Replace all dir separaters by spaces
	local DIR_LIST=${DIR_LIST//\// }
	local HEADER="Linux"
	local DIR=""
	for DIR in $DIR_LIST; do
		if [ -f $SYS_DIR/$DIR/base.title ]; then
			HEADER="$HEADER `cat $SYS_DIR/$DIR/base.title`"
		fi
		SYS_DIR=$SYS_DIR/$DIR
	done
	echo $HEADER
	echo "==============================================================================="
	echo ${PLACE}
	echo "-------------------------------------------------------------------------------"
}

function do_config_request () {
	# This creates a header
	# Prints the base.desc file or No section description
	# And then asks the user if it want to configure anything in this
	# section
	
	create_header $1
	if [ -f $DESCRIPTION_DIR_SYS/$1/base.desc ]; then
		cat $DESCRIPTION_DIR_SYS/$1/base.desc
	else
		echo "No section description for $1"
	fi
	echo
	echo -n "Do you want configure these settings [Y/N]"; read N
	if [ "x$N" = "xY" ]; then
		fstate `echo $1/*`
	fi
}

function fstate () {
	local ONE=""
	for ONE in $@; do
		if [ -f $ONE ]; then
			# So it is a file. Let's see if it is writable.
			# Only if it is we want to set values :)
			# test -w doesn't seem to work reliable on /proc
			local CURRENT_VALUE=`cat $ONE`
			echo "$CURRENT_VALUE" 2>/dev/null > $ONE
			if [ $? = 0 ]; then
				create_config $ONE
			fi
		fi
	done
}

function dir_loop () {
	# Let's look inside the dir what's there
	# And feed that to fstate to determine what to do with it
	local ONE=0
	local ALL=`echo $1/*`
	for ONE in $ALL; do
		if [ -d $ONE ]; then
			DIR_INDEX=$(($DIR_INDEX+1))
			DIR_DB[$DIR_INDEX]=$ONE
			DIRS="$DIRS $ONE"
		fi
	done
	loop_dir_loop $DIRS
}

function loop_dir_loop () {
	local DIR=""
	DIRS=""
	for DIR in $@; do
		dir_loop $DIR
	done
}

# Main

# Program vars
declare DIR_DB
DIR_INDEX=0
DIRS=""

# For testing purposes to jump into a branch:
if [ "x$1" != "x" ]; then
	PROC="$PROC/$1"
fi

echo -n "Creating DB... "
dir_loop $PROC
echo "done."

clear
echo "==============================================================================="
echo "Haring: Linux configuration"
echo "==============================================================================="
echo "Essential Linux terminology:"
echo "Jiffie: Internal timeunit for the kernel. On the i386 1/100s, on the"
echo "        Alpha 1/1024s. See the HZ define in /usr/include/asm/param.h for the"
echo "        exact value on your system."
echo
echo -n "<press any key to continue> "; read N

for (( I=1; I < ${#DIR_DB[*]}; I=$(($I+1)) )); do
	do_config_request ${DIR_DB[$I]}
done
